﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using BSF_Code_Library.Helpers;

namespace Able_Opus_Multi_Downloader
{
    class ActiveDownloadsPanel : Panel
    {
        public event EventHandler SelectionChanging;
        public event EventHandler SelectedChanged;
        public event EventHandler DownloadAdded;
        private int _lastSelectedIndex = -1;

        public ActiveDownloadsPanel()
        {
            this.BorderStyle = BorderStyle.FixedSingle;
            this.SetStyle(ControlStyles.Selectable, true);
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            base.OnKeyDown(e);

            if (e.KeyCode.Equals(Keys.Delete))
            {
                foreach (DownloadBar bar in SelectedDownloads)
                {
                    bar.StopDownload();
                    bar.Dispose();
                }
            }
        }

        private void OnSelectionChanged()
        {
            if (SelectionChanging != null)
            {
                SelectionChanging(this, EventArgs.Empty);
            }
        }

        private void OnSelectedChanged()
        {
            if (SelectedChanged != null)
            {
                SelectedChanged(this, EventArgs.Empty);
            }
        }

        private void OnDownloadAdded()
        {
            if (DownloadAdded != null)
            {
                DownloadAdded(this, EventArgs.Empty);
            }
        }

        public void AddDownload(string URL)
        {
            // If download exists, then select it and return
            foreach (DownloadBar bar in DownloadBars)
            {
                if (bar.DownloadLocation == URL)
                {
                    DeSelectAllDownloads();
                    bar.IsSelected = true;
                    return;
                }
            }

            DownloadBar downloadBar = new DownloadBar();
            downloadBar.Dock = DockStyle.Top;
            this.Controls.Add(downloadBar);
            downloadBar.BeginDownloadingFile(URL);

            OnDownloadAdded();
        }

        public void AddDownloadRange(string[] links)
        {
            this.SuspendLayout();

            foreach (string link in links)
            {
                bool alreadyExist = false;

                // If download exists, then select it and return
                foreach (DownloadBar bar in DownloadBars)
                {
                    if (link == bar.DownloadLocation)
                    {
                        DeSelectAllDownloads();
                        bar.IsSelected = true;
                        alreadyExist = true;
                    }
                }

                if (alreadyExist) continue;

                DownloadBar downloadBar = new DownloadBar();
                downloadBar.DownloadLocation = link;
                downloadBar.Dock = DockStyle.Top;
                this.Controls.Add(downloadBar);
            }

            foreach (DownloadBar bar in DownloadBars)
            {
                bar.BeginDownloadingFile();
            }

            this.ResumeLayout();

            OnDownloadAdded();
        }

        protected override void OnControlAdded(ControlEventArgs e)
        {
            base.OnControlAdded(e);
            DownloadBar downloadBar = (DownloadBar)e.Control;
            downloadBar.SelectionChanging +=new EventHandler(downloadBar_SelectionChanging);
            downloadBar.SelectedChanged +=new EventHandler(downloadBar_SelectedChanged);
        }

        private void downloadBar_SelectionChanging(object sender, EventArgs e)
        {
            bool ctrl = KeyState.IsKeyPressed(Keys.ControlKey);
            bool shift = KeyState.IsKeyPressed(Keys.Shift);

            if (!ctrl && !shift)
            {
                DeSelectAllDownloads();
            }
        }

        private void downloadBar_SelectedChanged(object sender, EventArgs e)
        {
            OnSelectedChanged();
        }

        public void DeSelectAllDownloads()
        {
            for (int i = 0; i < this.Controls.Count; i++)
            {
                DownloadBar downloadBar = this.Controls[i] as DownloadBar;

                if (downloadBar != null)
                {
                    downloadBar.IsSelected = false;
                }
            }
        }

        public void ClearAllInactiveDownloads()
        {
            foreach (DownloadBar bar in DownloadBars)
            {
                if (bar.IsInactive)
                {
                    bar.StopDownload();
                }
            }
        }

        public void StopAndClearAllDownloads()
        {
            foreach (DownloadBar bar in DownloadBars)
            {
                bar.StopDownload();
            }
        }

        public void StopAndClearSelected()
        {
            foreach (DownloadBar bar in SelectedDownloads)
            {
                bar.StopDownload();
            }
        }

        public void SetLastSelectedIndex(int index)
        {
            _lastSelectedIndex = index;
        }

        public void SelectItem(int index)
        {
            ((DownloadBar)this.Controls[index]).IsSelected = true;
        }

        #region Properties
        public DownloadBar[] DownloadBars
        {
            get
            {
                List<DownloadBar> barList = new List<DownloadBar>();

                for (int i = 0; i < this.Controls.Count; i++)
                {
                    barList.Add((DownloadBar)this.Controls[i]);
                }

                return barList.ToArray();
            }
        }

        public DownloadBar[] SelectedDownloads
        {
            get
            {
                List<DownloadBar> bars = new List<DownloadBar>();

                foreach (DownloadBar bar in DownloadBars)
                {
                    if (bar.IsSelected)
                    {
                        bars.Add(bar);
                    }
                }

                return bars.ToArray();
            }
        }

        public bool AllDownloadsCompleted
        {
            get
            {
                foreach (DownloadBar bar in DownloadBars)
                {
                    if (!bar.Completed) return false;
                }

                return true;
            }
        }

        public int TotalProgress
        {
            get
            {
                float progressSum = 0;

                DownloadBar[] bars = this.DownloadBars;
                foreach (DownloadBar bar in bars)
                {
                    progressSum += bar.Progress;
                }

                return (int)(progressSum / bars.Length + 0.5);
            }
        }

        public bool HasActiveDownload
        {
            get
            {
                foreach (DownloadBar bar in DownloadBars)
                {
                    if (!bar.IsInactive)
                    {
                        return true;
                    }
                }

                return false;
            }
        }
        #endregion
    }
}
