﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using Able_Opus_Multi_Downloader.Properties;

namespace Able_Opus_Multi_Downloader
{
    public partial class DownloadBar : UserControl
    {
        public bool Completed { get; set; }
        public string DownloadLocation { get; set; }

        private WebClient _webClient = new WebClient();
        private bool _isSelected;

        public event EventHandler SelectionChanging;
        public event EventHandler SelectedChanged;
        public event EventHandler DownloadCompleted;

        public DownloadBar()
        {
            InitializeComponent();
            _webClient.DownloadFileCompleted += new AsyncCompletedEventHandler(_webClient_DownloadFileCompleted);
            _webClient.DownloadProgressChanged += new DownloadProgressChangedEventHandler(_webClient_DownloadProgressChanged);
        }

        protected virtual void OnSelectionChanging()
        {
            if (SelectionChanging != null)
            {
                SelectionChanging(this, EventArgs.Empty);
            }
        }

        protected virtual void OnSelectedChanged()
        {
            if (SelectedChanged != null)
            {
                SelectedChanged(this, EventArgs.Empty);
            }
        }

        private void OnDownloadCompleted()
        {
            if (DownloadCompleted != null)
            {
                DownloadCompleted(this, EventArgs.Empty);
            }
        }

        public void BeginDownloadingFile(string URL)
        {
            try
            {
                DownloadLocation = URL;
                lblFilename.Text = Path.GetFileName(URL);
                string dest = Properties.Settings.Default.DownloadDir + @"\" + lblFilename.Text;
                _webClient.DownloadFileAsync(new Uri(URL), dest);
            }
            catch (UriFormatException ex)
            {
                StopDownload();
                this.Dispose();
                Misc.AddErrorLog(ex.Message, URL);
            }
            catch (ArgumentException ex)
            {
                StopDownload();
                this.Dispose();
                Misc.AddErrorLog(ex.Message, URL);
            }
            catch (Exception ex)
            {
                superProgressBar.BackColor = Color.DarkRed;
                lblStatus.Text = "Could not start";
                Misc.AddErrorLog(ex.Message, URL);
                IsInactive = true;
            }
        }

        public void BeginDownloadingFile()
        {
            try
            {
                lblFilename.Text = Path.GetFileName(DownloadLocation);
                string dest = Properties.Settings.Default.DownloadDir + @"\" + lblFilename.Text;
                _webClient.DownloadFileAsync(new Uri(DownloadLocation), dest);
            }
            catch (UriFormatException ex)
            {
                StopDownload();
                this.Dispose();
                Misc.AddErrorLog(ex.Message, DownloadLocation);
            }
            catch (ArgumentException ex)
            {
                StopDownload();
                this.Dispose();
                Misc.AddErrorLog(ex.Message, DownloadLocation);
            }
            catch (Exception ex)
            {
                superProgressBar.BackColor = Color.DarkRed;
                lblStatus.Text = "Could not start";
                Misc.AddErrorLog(ex.Message, DownloadLocation);
                IsInactive = true;
            }
        }

        private void _webClient_DownloadFileCompleted(object sender, AsyncCompletedEventArgs e)
        {
            ((WebClient)sender).Dispose();
            IsInactive = true;

            if (e.Cancelled) // Make sure partially downloaded files are deleted
            {
                string filePath = Settings.Default.DownloadDir + "\\" + lblFilename.Text;

                if (File.Exists(filePath))
                {
                    try
                    {
                        File.Delete(filePath);
                    }
                    catch (Exception ex)
                    {
                        Misc.AddErrorLog(ex.Message, DownloadLocation);
                    }
                }

                this.Dispose();
                return;
            }

            if (e.Error != null)
            {
                superProgressBar.BackColor = Color.DarkRed;
                lblStatus.Text = "Failed";
            }
            else
            {
                lblStatus.Text = "Completed";
                Completed = true;
                OnDownloadCompleted();
            }

            if (Settings.Default.AutoClear)
            {
                this.Dispose();
                return;
            }
        }

        private void _webClient_DownloadProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (lblStatus.Text == "Queued")
            {
                this.SendToBack();
                lblStatus.Text = "Downloading";
            }

            superProgressBar.Progress = e.ProgressPercentage;
        }

        private void lblFilename_DoubleClick(object sender, EventArgs e)
        {
            // Open file in explorer and select it
            string filePath = Settings.Default.DownloadDir + "\\" + lblFilename.Text;
            string argument = "/select, \"" + filePath + "\"";

            try
            {
                System.Diagnostics.Process.Start("explorer.exe", argument);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Cancels the async download associated with this control
        /// </summary>
        public void StopDownload()
        {
            // If download has been completed then dispose and return, in this case, calling CancelAsync will not raise
            // the OnDownloadCompleted event and we will not be able to dispose this in its handler
            if (IsInactive)
            {
                this.Dispose();
                return;
            }

            _webClient.CancelAsync();
        }

        private void Generic_MouseClick(object sender, MouseEventArgs e)
        {
            OnSelectionChanging();
            IsSelected = !_isSelected;
        }

        public bool IsInactive { get; private set; }

        public bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                _isSelected = value;

                if (!value)
                {
                    // Set backcolors to default color
                    Color color = Color.FromKnownColor(KnownColor.ControlLightLight);
                    this.BackColor = color;
                    superProgressBar.BackColor = Color.YellowGreen;
                }
                else
                {
                    // Set backcolors to users highlight color
                    Color color = Color.FromKnownColor(KnownColor.Highlight);
                    this.BackColor = color;
                    superProgressBar.BackColor = color;
                }

                OnSelectedChanged();
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            StopDownload();
        }

        public float Progress
        {
            get { return superProgressBar.Progress; }
        }
    }
}
